@@ -9,7 +9,7 @@ import sys
 from gitflow.core import GitFlow
 from gitflow.exceptions import (GitflowError, MultipleReviewRequestsForBranch,
                                 NoSuchBranchError, AncestorNotFound, EmptyDiff,
-                                PostReviewError)
+                                PostReviewError, SubmitReviewError)
 
 class ReviewRequestLimitError(GitflowError):
     def __str__(self):
@@ -78,9 +78,21 @@ class BranchReview(object):
 
     def post(self, story, summary_from_story=True):
         assert self._rev_range
+
+        def to_unicode(s):
+            try:
+                return unicode(s)
+            except UnicodeDecodeError:
+                return unicode(s, encoding='utf8')
+
+        def to_string(us):
+            try:
+                return str(us)
+            except UnicodeEncodeError:
+                return us.encode(encoding='utf8')
+
         cmd = ['rbt', 'post',
-               '--branch', self._branch,
-               '--revision-range={0[0]}:{0[1]}'.format(self._rev_range)]
+               '--branch', self._branch]
 
         self._check_for_existing_review()
 
@@ -93,8 +105,8 @@ class BranchReview(object):
                         "%s%n%n"
                         "%b",
                     '{0[0]}...{0[1]}'.format(self._rev_range)]
-        desc_prefix = '> Story being reviewed: {0}\n'.format(story.get_url())
-        desc = desc_prefix + '\nCOMMIT LOG\n' + sub.check_output(desc_cmd)
+        desc_prefix = u'> Story being reviewed: {0}\n'.format(story.get_url())
+        desc = desc_prefix + u'\nCOMMIT LOG\n' + to_unicode(sub.check_output(desc_cmd))
 
         if summary_from_story:
             summary = story.get_name()
@@ -113,8 +125,8 @@ class BranchReview(object):
             for line in lines:
                 if line.startswith('> Story being reviewed'):
                     break
-                kept_desc.append(line)
-            desc = '\n'.join(kept_desc) + '\n' + desc
+                kept_desc.append(to_unicode(line))
+            desc = u'\n'.join(kept_desc) + u'\n' + to_unicode(desc)
 
         if self._rid:
             sys.stdout.write('updating %s ... ' % str(self._rid))
@@ -123,8 +135,10 @@ class BranchReview(object):
         else:
             sys.stdout.write('new review ... ')
 
-        cmd.append('--summary=' + str(summary))
-        cmd.append('--description=' + str(desc))
+        cmd.append(u'--summary=' + to_unicode(summary))
+        cmd.append(u'--description=' + to_unicode(desc))
+        cmd.extend([str(rev) for rev in self._rev_range])
+        cmd = [to_string(itm) for itm in cmd]
 
         p = sub.Popen(cmd, stdout=sub.PIPE, stderr=sub.PIPE)
         (outdata, errdata) = p.communicate()
@@ -164,7 +178,19 @@ class BranchReview(object):
         assert self._status
         if self._status == 'submitted':
             return
-        self._update(status='submitted')
+
+        cmd = ['rbt', 'close', '--close-type', 'submitted', str(self.get_id())]
+        p = sub.Popen(cmd, stdout=sub.PIPE, stderr=sub.PIPE)
+        (outdata, errdata) = p.communicate()
+
+        if p.returncode != 0:
+            print('>>>> rbt error output')
+            print(errdata)
+            print('<<<< rbt error output')
+            print('If the error output is not sufficient, execute')
+            print('\n    $ rbt close --debug --close-type submitted {0}\n'.format(self.get_id()))
+            print('and see what happens.')
+            raise SubmitReviewError('Failed to submit review request.')
 
     def is_accepted(self):
         assert self._rid
@@ -254,7 +280,7 @@ class Release(object):
                 print('    ' + str(e))
                 err = e
         if err is not None:
-            raise SystemExit('An error detected, aborting...')
+            raise SystemExit(str(err))
         if not ignore_missing_reviews and len(self._reviews) != reviews_expected:
             raise SystemExit('Some stories have not been reviewed yet,' \
                     ' aborting...')
